/*********************************************************************
 *
 * Use Knuth's method to estimate the search tree size for given permutation
 *
 ********************************************************************/

#include "prog.h"

double
analyse (sample)
     int sample;
{
  /* Guess number of trials required for given pi
   * assuming setup(SA) has been called)
   * returns the average result over sample trials
   */
  int rho;
  register short rp, n, t, np;
  register int ii, sample_no, numrels;
  int tnum;
  short tval;
  double result, mult;
  double total;

  trials = 0;
  total = 0;
  for (sample_no = 1; sample_no <= sample; sample_no++) {
    result = 0;
    mult = 1;
    n = 0;
    t = 1;
    for (;;) {			/* loop analyse_do1: */
      if (n == SA) {
	goto analyse_od1;
      }
      n++;
      tnum = 0;
      for (t = 1; t <= SB; t++) {
	trials = trials + 1;

#include "relpres.c"
/* Uses np, rho, ii and numrels to calculate relpres(psi,t,n)
 * Returns result in rp
 */
	if (rp == 1) {
	  tvals[tnum] = t;
	  tnum++;
	}
      }
      /* add weighted no. of relpres calls made: */
      result = result + mult * SB;
      if (tnum > 0) {
	mult = mult * tnum;
	tval = (random () % tnum);	/* pick a random next move */
	psi[n] = tvals[tval];
      } else {
	/* no tvals, so finish this guess */
	goto analyse_od1;
      }
    }
  analyse_od1:;
    total += result;
  }				/* next sample_no */
  /* Return average of all results */
  return (total / sample);
}				/* End of analyse(sample) */

void
do_analyse ()
{
  double result;
  int a, b, samp, retry, tested;
  double known_trials;
  double current, new;

  (void) printf ("\n--- Using new analysis method based on Knuth ---\n");
  samp = SAMPLES;
  retry = 0;
  randompi ();
  setup (SA);
  current = analyse (samp);
  /* The basic method is to keep munging the permutation, using the Knuth analysis
   * to see if a better perm has been found. Once the Knuth analysis is "good enough"
   * i.e. less than half the total trials so far, we do a trial calculation with
   * this permuation.
   */
  a = 1;
  b = 1;
  tested = 0;
  for (;;) {
    random_pair (&a, &b);
    insert (a, b);		/* insert x in y's place and shift up/down */
    setup (SA);
    new = analyse (samp);
    total_trials += trials;
    tested++;
    if (new < current) {
      /* have a better perm */
      (void) printf ("%6i T: %e, C: %e, N: %e\n", tested, total_trials, current, new);
      current = new;
      retry = 0;
    } else {
      /* restore old perm */
      insert (b, a);
      /* Every 20 failures we recalculate the current value.
       * This is in case we accidentally hit an artificially low estimate.
       */
      retry++;
      if (retry >= 10) {
	setup (SA);
	new = analyse (samp);
	/* If the old and new estimates differ by more than 50%
	 * then we increase the sample size
	 */
	if (fabs (new - current) > ((new + current) / 4)) {
	  samp = samp * MULT / 100;
	  if (samp > MAXSAMPLES) {
	    samp = MAXSAMPLES;
	  }
	  current = analyse (samp);
	  (void) printf ("New samp: %3i, new current: %e\n", samp, current);
	}
	retry = 0;
      }
    }
    if (current * 4 < total_trials) {
      /* current perm seems to be feasable: */
      (void) printf ("\n--- Now doing trial calculation with this perm ---\n");
      result = calculate (1, SA, total_trials / 2);
      if (trials <= total_trials / 2) {	/* this perm succeeded */
	(void) printf ("maps = %16.0f,\t", result);
	(void) printf ("in %16.0f trials\n", trials);
	printinfo (1, SA, trials, result);
	break;			/* terminate do_analyse loop */
      } else {
	(void) printf ("Trial calculation failed - the search continues...\n");
	/* We know that the search space is > trials
	 * so keep increasing the sample until we get a guess
	 * which is > trials
	 */
	known_trials = trials;
	total_trials += trials;
	do {
	  samp = samp * MULT / 100;
	  if (samp > MAXSAMPLES) {
	    samp = MAXSAMPLES;
	  }
	  current = analyse (samp);
	  (void) printf ("New samp: %3i, new current: %e\n", samp, current);
	} while ((current < known_trials) && (samp < MAXSAMPLES));
      }
    }				/* fi (current * 4 < total_trials) */
  }				/* End new analyse loop */
}				/* End of do_analyse */
