/*********************************************************************
 *
 * Translation program for relation preserving mappings program.
 *
 * Input is via stdin in the following format:
 * Asize
 * Bsize
 * no of relations
 * list of A triples in form:
 * i j k
 * meaning RA[i,j,k]=1
 * then a triple:
 * -1 -1 -1
 * then list of B triples in same format.
 * finally another triple:
 * -1 -1 -1
 *
 * Output is via stdout in the following format: (suitable for prog)
 * Asize
 * Bsize
 * no of relations
 * list of A relations (matrices)
 * list of B relations
 *
 ********************************************************************/

#include <stdio.h>
#include <malloc.h>		/* for malloc(size) */
#include <varargs.h>		/* for error function */

extern int scanf ();
extern int printf ();
extern int fprintf ();
extern int vfprintf ();
extern void exit ();


/* error should be called like:
 *    error(function_name, format, arg1, arg2...);
 *  Note: function_name and format cannot be declared
 *  separately because of the definition of varargs.
 */
 /*VARARGS0*/
void
error (va_alist)
     va_dcl
{
  va_list args;
  char *fmt;

  va_start (args);
  /* print name of function causing error */
  (void) fprintf (stderr, "ERROR in %s: ", va_arg (args, char *));
  fmt = va_arg (args, char *);
  /* print out remainder of message */
  (void) vfprintf (stderr, fmt, args);
  va_end (args);
  (void) exit (1);
}

short
***
allocate_3D (x, y, z)
     int x, y, z;
{
  int i, j;
  short ***result;
  /* allocate memory for a 3D short array of size 0..x by 0..y by 0..z
   * returns pointer to the array:
   */
  result = (short ***) malloc ((unsigned) ((x + 1) * sizeof (*result)));
  for (i = 0; i <= x; i++) {
    result[i] = (short **) malloc ((unsigned) ((y + 1) * sizeof (**result)));
    for (j = 0; j <= y; j++) {
      result[i][j] = (short *) malloc ((unsigned) ((z + 1) * sizeof (short)));
    }
  }
  return (result);
}

void
readtriples (A, x, y, z, drop)
     short ***A;
     int x, y, z, drop;
{				/* read data from stdin */
  int i, j, k;
  int ret;

  for (;;) {
    ret = scanf (" %i %i %i", &i, &j, &k);
    if (ret == EOF) {
      error ("readtriples", "Premature end of file!\n");
    }
    if (i == -1)
      break;
    if (i != drop) {
      if ((drop > 0) && (i > drop)) {
	i = i - 1;
      }
      /* Check the ranges */
      if ((i < 1) || (i > x) || (j < 1) || (j > y) || (k < 1) || (k > z)) {
	error ("readtriples", "Triple out of range: %i %i %i\n", i, j, k);
      }
      A[i][j][k] = 1;
    }
  }
}				/* End of readtriples (A, x, y, z) */


void
printarray (A, x, y, z)
     short ***A;
     int x, y, z;
{
  int i, j, k;

  for (i = 1; i <= x; i++) {
    for (j = 1; j <= y; j++) {
      for (k = 1; k <= z; k++) {
	(void) printf ("%s ", (A[i][j][k] == 1) ? "1" : "0");
      }
      (void) printf ("\n");
    }
    (void) printf ("\n");
  }
  (void) printf ("\n");
}				/* End of printarray(A, x, y, z) */

int
main ()
{
  int R, SA, SB, drop;
  short ***RA, ***RB;

  /* Read in the no of relations and sizes of A and B, and the dropped rel */
  (void) scanf (" %i %i %i %i", &R, &SA, &SB, &drop);
  if (drop > 0) {
    R = R - 1;
  }
  /* Allocate the arrays */
  RA = allocate_3D (R, SA, SA);
  RB = allocate_3D (R, SB, SB);
  /* Read triples for A array (until -1 -1 -1 is read) */
  readtriples (RA, R, SA, SA, drop);
  /* Read triples for B array (until -1 -1 -1 is read) */
  readtriples (RB, R, SB, SB, drop);

  /* Print no of relations and sizes of A and B */
  (void) printf ("%i\n%i\n%i\n", R, SA, SB);
  /* Print A array */
  printarray (RA, R, SA, SA);
  printarray (RB, R, SB, SB);

  return (0);
}				/* End of main */
